import * as React from 'react';
import { OFRest } from '../../components/Rest';
import Dialog from 'material-ui/Dialog';
import FlatButton from 'material-ui/FlatButton';
import RaisedButton from 'material-ui/RaisedButton';
import { Card, CardActions, CardHeader, CardText } from 'material-ui/Card';
import * as Utils from '../../components/Utils';
import DropDownMenu from 'material-ui/DropDownMenu';
import MenuItem from 'material-ui/MenuItem';
import AppBar from 'material-ui/AppBar';
import Checkbox from 'material-ui/Checkbox';
import TextField from 'material-ui/TextField';
import { RadioButton, RadioButtonGroup } from 'material-ui/RadioButton';
import {
    Table,
    TableBody,
    TableHeader,
    TableHeaderColumn,
    TableRow,
    TableRowColumn,
} from 'material-ui/Table';

const customContentStyle = {
    width: '90%',
    maxWidth: 'none',
};

export class SensorConfigPopup extends React.Component<any, any> {
    constructor(props: any, context: any) {
        super(props, context);
        this.state = {
            crawler: null,
            sensorItems: []
        };
    }

    handleClose = () => {
        this.props.onCancel();
    }

    handleCrawlerSelect = (c: any) => {
        //getting items by target&crawler type
        var items = [
            {
                "id": 0,
                "category": "cpu",
                "name": "cpu.usage",
                "description": "cpu usage ",
            },
            {
                "id": 1,
                "category": "cpu",
                "name": "cpu.free",
                "description": "cpu free ",
            },
            {
                "id": 2,
                "category": "mem",
                "name": "mem.free",
                "description": "mem free ",
            },
        ];

        this.setState({
            crawler: c,
            sensorItems: items,
        });
    }

    handleSensorItemsSelect = (items: any) => {
        console.log(items);
    }

    render() {
        const actions = [
            <FlatButton
                label="Cancel"
                primary={true}
                onTouchTap={this.handleClose}
            />,
            <RaisedButton
                label="Submit"
                primary={true}
                onTouchTap={this.handleClose}
            />,
        ];

        return (
            <div>
                <Dialog
                    title="Sensor Configuration"
                    actions={actions}
                    modal={true}
                    contentStyle={customContentStyle}
                    open={this.props.open}
                >

                    <Card>
                        <CardHeader
                            title={this.props.target.vendorName}
                            subtitle={Utils.int2ip(this.props.target.ip)}
                            actAsExpander={true}
                            showExpandableButton={true}
                        />
                        <CardText expandable={true}>
                            <div>Discovered At : </div>
                            <div>Target Type : </div>
                            <div>Kind : </div>
                            <div>Name : </div>
                            <div>Version : </div>
                            <div>Port : </div>
                            <div>Port Type : </div>
                        </CardText>
                    </Card>

                    <Card>
                        <CrawlerSelector target={this.props.target} onChange={this.handleCrawlerSelect} />
                    </Card>
                    <Card>
                        <SensorItemSelector items={this.state.sensorItems} onChange={this.handleSensorItemsSelect} />
                    </Card>
                </Dialog>
            </div>
        );
    }
}


export class CrawlerSelector extends React.Component<any, any> {
    constructor(props: any, context: any) {
        super(props, context);
        this.state = {
            value: -1,
            crawlers: [],
            authOpen: false,
            authInputs: [],
        };
    }

    componentWillMount() {

        let os: OFRest = new OFRest("Crawler", "List", null);
        var that: any = this;
        os.Call().then(function (res) {
            return res.json();
        }).then(function (json) {
            let list = JSON.parse(json.resultStr);
            that.setState({
                crawlers: list,
            });
        }).catch(function (err) {
            console.log(err);
        });


    }

    handleCrawlerChange = (event: any, index: any, v: number) => {
        this.setState({
            value: v,
        });
        if (v === -1) {
            return;
        }
        let selectedCrawler = this.state.crawlers[v];
        this.props.onChange(selectedCrawler);
        this.checkAuth(selectedCrawler);
    }

    checkAuth = (c: any) => {
        //Todo. validate auth
        this.setState({
            authOpen: true,
        });

        let os: OFRest = new OFRest("Crawlerinputitemmapping", "List", c);
        var that: any = this;
        os.Call().then(function (res) {
            return res.json();
        }).then(function (json) {
            var list = JSON.parse(json.resultStr);
            that.setState({
                authInputs: list,
            });
        }).catch(function (err) {
            console.log(err);
        });
    }


    getCrawlerTitle() {
        if (this.state.value === -1) {
            return "";
        }
        var c = this.state.crawlers[this.state.value];
        return "[" + c.crawlerType + "] " + c.description;
    }

    renderAuthRequester = () => {
        if (this.state.authInputs.length === 0) {
            return <div></div>;
        }
        return <div> Connection Info.  {
            this.state.authInputs.map((data: any, index: number) => {
                return <div key={index}>{this.renderInputs(data)}</div>;
            })
        }</div>;
    }

    renderInputs = (data: any) => {
        let input = data.crawlerInputItem;
        if (input.dataType === "String") {
            return <div><TextField
                hintText={input.description}
                floatingLabelText={input.name}
                underlineShow={true}
            //value={this.state.email}
            // onChange={(e, newValue) => this.setState({ email: newValue })}
            /></div>;
        }
        else if (input.dataType === "Select") {
            let values = input.description.split('||');
            let radios = Array();
            for (var i = 0; i < values.length; i++) {
                radios.push(<RadioButton key={i} value={i} label={values[i]} />);
            }
            return <div>
                <RadioButtonGroup name={input.name} defaultSelected="not_light">
                    {radios}
                </RadioButtonGroup>
            </div>;
        }
    }

    handleExpand = (b: boolean) => {
        this.setState({
            authOpen: b,
        });
    }

    render() {
        return (
            <div>
                <DropDownMenu value={this.state.value} onChange={this.handleCrawlerChange} style={{ width: 300 }}>
                    <MenuItem value={-1} primaryText="Choose a Crawler type" />
                    {this.state.crawlers.map((crawler: any, index: number) => {
                        return (<MenuItem
                            value={index}
                            primaryText={crawler.name}
                            key={index}
                        />);
                    })}

                </DropDownMenu>
                <Card expanded={this.state.authOpen} onExpandChange={this.handleExpand.bind(this)}>
                    <CardHeader
                        title={this.getCrawlerTitle()}
                        subtitle=''
                        actAsExpander={true}
                        showExpandableButton={true}
                    />
                    <CardText expandable={true} >
                        {this.renderAuthRequester()}
                    </CardText>
                </Card>

            </div>
        );
    }
}


export class SensorItemSelector extends React.Component<any, any> {

    constructor(props: any, context: any) {
        super(props, context);
        this.state = {
            selected: []
        };
    }


    isSelected = (index: any) => {
        return this.state.selected.indexOf(index) !== -1;
    }

    handleRowSelection = (selectedRows: any) => {
        this.setState({
            selected: selectedRows,
        });
        this.props.onChange(selectedRows);
    }

    render() {
        return (
            <Table onRowSelection={this.handleRowSelection} multiSelectable={true}>
                <TableHeader>
                    <TableRow>
                        <TableHeaderColumn>Category</TableHeaderColumn>
                        <TableHeaderColumn>Name</TableHeaderColumn>
                        <TableHeaderColumn>Description</TableHeaderColumn>
                    </TableRow>
                </TableHeader>
                <TableBody>

                    {this.props.items.map((row: any, index: number) => (
                        <TableRow key={index} selected={this.isSelected(index)}>
                            <TableRowColumn>{row.category}</TableRowColumn>
                            <TableRowColumn>{row.name}</TableRowColumn>
                            <TableRowColumn>{row.description}</TableRowColumn>
                        </TableRow>
                    ))}
                </TableBody>
            </Table>
        );
    }
}

